#include "geosim.h"
#include "gspriv.h"
#include "ietypes.h"
#include "infprocs.h"
#include "miscwind.h"
#include "aliasw.h"
#include "rootw.h"
#include "messagew.h"
#include "ifw.h"
#include "functnw.h"
#include "windfunc.h"
#include "vwinds.h"
#include "windprop.h"
#include "objprop.h"
#include "plabw.h"
#include "palettew.h"
IEIconfig iestate;

IntrFunction FuncNames[] = {
	{"redrawf", redrawfunc},
	{"new", new_interface},
	{"open", open_interface},
	{"process_key", (boolean(*)())process_key},
	{"save_interface", save_interface},
	{"ask_close_interface", ask_close_interface},
	{"ask_revert_interface", ask_revert_interface},
	{"ask_quit", ask_quit},
	{"action_confirmed", action_confirmed},
	{"action_denied", action_denied},
	{"action_canceled", action_canceled},
	{"quit", quit},
	{"edit_misc", edit_misc},
	{"edit_aliases", edit_aliases},
	{"edit_messages", edit_messages},
	{"edit_functions", edit_functions},
	{"edit_palette", edit_palette},
	{"edit_interface", edit_interface},
	{"edit", null_func},
	{"file", null_func},
	{"SetWindBstore", null_func},
	{"null_func", null_func},
	{"setpropactive", null_func},
	{"setproplabpos", null_func},
	{"revert_interface", null_func},
	{"misc_sel_redrawfunc", misc_sel_redrawfunc},
	{"misc_inc_speedup", misc_inc_speedup},
	{"misc_dec_speedup", misc_dec_speedup},
	{"misc_toggle_diag", misc_toggle_diag},
	{"misc_col_up", misc_col_up},
	{"misc_col_down", misc_col_down},
	{"alias_pick", alias_pick},
	{"color_pick", color_pick},
	{"alias_scroll", alias_scroll},
	{"color_scroll", color_scroll},
	{"edit_alias_field", edit_alias_field},
	{"new_alias", new_alias},
	{"delete_alias", delete_alias},
	{"alias_up", alias_up},
	{"alias_down", alias_down},
	{"col_up", col_up},
	{"col_down", col_down},
	{"message_up", message_up},
	{"message_down", message_down},
	{"message_scroll", message_scroll},
	{"message_pick", message_pick},
	{"edit_message_name", edit_message_name},
	{"edit_message_text", edit_message_text},
	{"new_message", new_message},
	{"delete_message", delete_message},
	{"idrag", idrag},
	{"ikey", (boolean(*)())ikey},
	{"misc_color_pick", misc_color_pick},
	{"misc_color_scroll", misc_color_scroll},
	{"misc_set_highlight", misc_set_highlight},
	{"misc_set_border", misc_set_border},
	{"misc_set_tlcolor", misc_set_tlcolor},
	{"misc_set_brcolor", misc_set_brcolor},
	{"function_pick", function_pick},
	{"function_scroll",function_scroll },
	{"function_up", function_up},
	{"function_down", function_down},
	{"edit_function_name", edit_function_name},
	{"function_toggle_active", function_toggle_active},
	{"function_inc_dtime", function_inc_dtime},
	{"function_dec_dtime", function_dec_dtime},
	{"function_toggle_mouseoff", function_toggle_mouseoff},
	{"delete_function", delete_function},
	{"new_function", new_function},
	{"quit_visual", (boolean(*)())RemoveInterfaceWind},
	{"vwind_pick", vwind_pick},
	{"vwind_scroll", vwind_scroll},
	{"vwind_up", vwind_up},
	{"vwind_down", vwind_down},
	{"vwind_edit_prop", vwind_edit_prop},
	{"show_winds", show_winds},
	{"vwind_ok", vwind_ok},
	{"wname_prop", wname_prop},
	{"set_wind_bstore", set_wind_bstore},
	{"set_wind_x", set_wind_x},
	{"set_wind_y", set_wind_y},
	{"set_wind_wd", set_wind_wd},
	{"set_wind_ht", set_wind_ht},
	{"set_wind_bsize", set_wind_bsize},
	{"toggle_wind_active", toggle_wind_active},
	{"set_wind_fg", set_wind_fg},
	{"set_wind_bg", set_wind_bg},
	{"set_wind_bdr", set_wind_bdr},
	{"w_col_pick", w_col_pick},
	{"w_col_scroll", w_col_scroll},
	{"w_col_btn", w_col_btn},
	{"w_ok", w_ok},
	{"w_cancel", w_cancel},
	{"w_delete", w_delete},
	{"set_prop_x", set_prop_x},
	{"set_prop_y", set_prop_y},
	{"set_prop_wd", set_prop_wd},
	{"set_prop_ht", set_prop_ht},
	{"set_prop_col1", set_prop_col1},
	{"set_prop_col2", set_prop_col2},
	{"set_prop_col3", set_prop_col3},
	{"set_prop_col4", set_prop_col4},
	{"prop_col_pick", prop_col_pick},
	{"prop_col_scroll", prop_col_scroll},
	{"prop_col_btn", prop_col_btn},
	{"prop_func_pick", prop_func_pick},
	{"prop_func_scroll", prop_func_scroll},
	{"prop_func_btn", prop_func_btn},
	{"toggle_prop_bool1", toggle_prop_bool1},
	{"toggle_prop_bool2", toggle_prop_bool2},
	{"pname_prop", pname_prop},
	{"ptext_prop", ptext_prop},
	{"set_prop_func", set_prop_func},
	{"prop_ok", prop_ok},
	{"prop_cancel", prop_cancel},
	{"prop_delete", prop_delete},
	{"set_prop_active", set_prop_active},
	{"set_vis_state", set_vis_state},
	{"set_prop_labpos", set_prop_labpos},
	{"pfunc_prop", pfunc_prop},
	{"plab_scroll", plab_scroll},
	{"plab_pick", plab_pick},
	{"plab_btn", plab_btn},
	{"plab_name", plab_name},
	{"plab_text", plab_text},
	{"plab_func", plab_func},
	{"plab_toggle", plab_toggle},
	{"plab_new", plab_new},
	{"plab_delete", plab_delete},
	{"col_btn", col_btn},
	{"col_new", col_new},
	{"col_delete", col_delete},
	{"col_red", col_red},
	{"col_green", col_green},
	{"col_blue", col_blue},
	{"col_pick", col_pick},
	{"col_scroll", col_scroll},
	{"col_name_edit", col_name_edit},
	{"col_display", col_display},
	{"dump_screen", dump_screen},
	{"vis_save", vis_save},
};

int NumFuncs = sizeof(FuncNames) / sizeof(IntrFunction);
