#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "geosim.h"
#include "gspriv.h"

#include "ietypes.h"
#include "iefile.h"
#include "editrec.h"
#include "iemisc.h"

/* +++ error - const * and linestr */

/*internal prototypes*/
void write_palette(FILE *fptr);
void write_aliases(FILE *fptr);
void write_highlight(FILE *fptr);
void write_speedup(FILE *fptr);
void write_buttonstuff(FILE *fptr);
void write_colorlist(FILE *fptr);
void write_redrawfunc(FILE *fptr);
void write_message(FILE *fptr, IEMessage *nextmessage);
void write_function(FILE *fptr, IEFunction *nextfunction);
void write_popup(FILE *fptr, IEPopup *nextpopup);
void write_menu_item(FILE *fptr, IEMenuItem *nextmenuitem);
void write_window(FILE *fptr, IEWindow *nextwindow);
void write_button(FILE *fptr, IEButton *nextbutton);
void write_dragarea(FILE *fptr, IEDragArea *nextdragarea);
void write_graphic(FILE *fptr, IEGraphic *nextgraphic);
void write_field(FILE *fptr, IEField *nextfield);
void free_popupitems(IEPopup *p);
void free_buttons(IEWindow *w);
void free_dragareas(IEWindow *w);
void free_graphics(IEWindow *w);
void free_fields(IEWindow *w);
void error(char *message, char *info);
void ieerror(char *msg);
/**/

int curiecolorlist;
int curiealias;
int pindex;

static boolean fileOpenSuccess;
extern IEIconfig iestate;

/*char*/ GSColor IEInterface_Palette[3*256];
char *IEColorlist[257];
char *IEAlias[MAXALIASES+1], *IEAcolor[MAXALIASES+1];

/* Here's how palette handling works:
	When we're editing non-visual stuff, we use our colors (ie's).
	When we're editing visual elements, we use their colors
	(we load their color palette into ours).
*/

extern BigString filename;

 HugeString ieLineBuf;
 int ieLineNumber;
 
 extern FILE *iein;
void ieparse();
void ierestart(FILE *);

void reset_state()
{
	int i;
	EditRec *newrec;
	IEAlias[0] = 0;
	*iestate.highlight = '\0';
	iestate.speedup = -1;
	*iestate.buttonborder = '\0';
	*iestate.buttonTLcolor = '\0';
	*iestate.buttonBRcolor = '\0';
	iestate.diagonals = -1;
	*iestate.redrawfunc = '\0';
	*iestate.colorlist[0][0] = '\0';  /* free this? */
	
	/* free colorlist */
	for (i=0; IEColorlist[i] != NULL && *IEColorlist[i] != '\0'; i++) {
		free (IEColorlist[i]);
		IEColorlist[i] = NULL;
		}
	
	iestate.msglisthead = iestate.msglisttail = NULL;
	iestate.funclisthead = iestate.funclisttail = NULL;
	iestate.winlisthead = iestate.winlisttail = NULL;
	iestate.popuplisthead = iestate.popuplisttail = NULL;
	IEAlias[0] = IEAcolor[0] = NULL;
}
	

int read_interface(BigString fname)
{
	FILE *fptr;
	HugeString linestr;
	BigString token;
	BigString strarg1, strarg2;
	static bool first_time = TRUE;

	if ((fptr = GSopen(fname, "r")) == NULL)	{
		error("Can't open file.", fname);
		return(FALSE);
		}
		
	/*  file state should be "closed"  */
	fileOpenSuccess = TRUE;
	if (first_time == TRUE) {
		iein = fptr;
		first_time = FALSE;
		}
	else
		ierestart(fptr);
	curiecolorlist=0;
	curiealias=0;
	
	pindex=NUM_STD_COLORS + NUM_DFLT_COLORS;
	init_default_palette(IEColorlist, (char *) IEInterface_Palette);
	ieparse();
	fclose(fptr);
	/*GSolog("\nRead all of the interface file.\n\n\n");*/
	return fileOpenSuccess;
}

/* parser support functions - called only from parser */

boolean get_palette_entry(char *name, int r, int g, int b)
{
	int p;
	p = GSiselword(name, IEColorlist);
	if (p == -1) {	/* not found */
		p = pindex;
		pindex++;
		if (pindex > 256) {
			ieerror("Too many colors.\n");
			return FALSE;
			}
		IEColorlist[p] = name;
		IEColorlist[p+1] = NULL;
		}
	if (p >= NUM_STD_COLORS) {
		IEInterface_Palette[p*3] = (char) r;
		IEInterface_Palette[p*3+1] = (char) g;
		IEInterface_Palette[p*3+2] = (char) b;
		}
	return TRUE;
}

boolean get_function(char *name, int dtime, int active, int mouseoff)
{
	IEFunction *newfunc;
	
	if ((newfunc = (IEFunction *) malloc (sizeof(IEFunction))) == NULL)	{
		ieerror("Out of memory for function record.\n");
		return FALSE;
		}
	if (iestate.funclisthead == NULL)	
		iestate.funclisthead = iestate.funclisttail = newfunc;
	else {
		iestate.funclisttail->nextfunc = newfunc;
		}
	iestate.funclisttail = newfunc;
	newfunc->nextfunc = NULL;
	strcpy(newfunc->name, name);
	newfunc->dtime = dtime;
	newfunc->active = active;
	newfunc->mouseoff = mouseoff;
	free(name);	
	
	return TRUE;
}	


boolean get_label(int sz, int x, int y, char *col, char *text, char *back)
{
	IEGraphic *newgraphic;
	
	if ((newgraphic = (IEGraphic *) malloc(sizeof(IEGraphic))) == NULL)	{
		ieerror("No memory for label.");
		return(FALSE);
		}
	
	if ((newgraphic->gr.text = malloc(BIGSTRINGLEN+1)) == NULL) {
		ieerror("No memory for label text.\n");
		return FALSE;
		}
		
	strcpy(newgraphic->gr.text, text);
	strcpy(newgraphic->grcol, col);
	
	if (back)
		strcpy(newgraphic->grshadowcol, back);
	else
		strcpy(newgraphic->grshadowcol, "NONE");
	
	newgraphic->gr.type = (sz == 1)? GR_LABEL : GR_LABEL_SMALL;
	
	set_label_dims(newgraphic, x, y);
	
	if (iestate.winlisttail->firstgraphic == NULL)
		iestate.winlisttail->firstgraphic = newgraphic;
	else
		iestate.winlisttail->lastgraphic->nextgraphic = newgraphic;
	newgraphic->prevgraphic = iestate.winlisttail->lastgraphic;
	iestate.winlisttail->lastgraphic = newgraphic;
	newgraphic->nextgraphic = NULL;
	
	free(col);
	free(text);
	if (back)
		free(back);
		
	return TRUE;
}

boolean get_plabel(char *name, int stat, char *func, char *text, int checked)
{
	IEMenuItem *newmenuitem;
	IEPopup *curpopup = iestate.popuplisttail;
	
	int len;
	
	if (curpopup == NULL)	{
		ieerror("No active popup for plabel.");
		return(FALSE);
		}
		
	if ((newmenuitem = (IEMenuItem *) malloc(sizeof(IEMenuItem))) == NULL)	{
		ieerror("No memory for menu item.");
		return(FALSE);
		}
	
	strcpy(newmenuitem->name, name);
	newmenuitem->active = (MenuItemStatusType) stat;
	strcpy(newmenuitem->funcname, func);
	strcpy(newmenuitem->text, text);
	newmenuitem->checked = checked;
	
	GSsetfont(LARGE);
	len = GStextwidth(text) + 2*POPUPOFFSET;
	if (curpopup->checklist)
		len += CHECKWID;
	if (len > curpopup->wd)
		curpopup->wd = len;
	if (newmenuitem->active != INVISIBLE)
		curpopup->ht += TSIZE;
	
	if (curpopup->itemlisthead == NULL)
		curpopup->itemlisthead = newmenuitem;
	else
		curpopup->itemlisttail->nextmenuitem = newmenuitem;
	curpopup->itemlisttail = newmenuitem;
	newmenuitem->nextmenuitem = NULL;
	
	free(name);
	free(func);
	free(text);
	
	return TRUE;
}

boolean get_popup(char *title, int x, int y, char *efg, char *dfg, 
							char *bg, char *bcolor, int checklist, char *func)
{
	IEPopup *newpopup;
	if ((newpopup = (IEPopup *) malloc(sizeof(IEPopup))) == NULL)	{
		ieerror("No memory for popup.");
		return FALSE;
		}
	strcpy(newpopup->title, title);
	newpopup->x = x;
	newpopup->y = y;
	strcpy(newpopup->efg, efg);
	strcpy(newpopup->dfg, dfg);
	strcpy(newpopup->bg,  bg);
	strcpy(newpopup->bcolor, bcolor);
	newpopup->checklist = checklist;
	if (func)
		strcpy(newpopup->function, func);
	else
		newpopup->function[0] = '\0';
	newpopup->itemlisthead = newpopup->itemlisttail = NULL;
	
	if (iestate.popuplisthead == NULL)
		iestate.popuplisthead = newpopup;
	else 
		iestate.popuplisttail->nextpopup = newpopup;
	newpopup->prevpopup = iestate.popuplisttail;
	iestate.popuplisttail = newpopup;
	newpopup->nextpopup = NULL;
	
	newpopup->visibleineditor = TRUE;
	newpopup->wd = 0;
	newpopup->ht = TSIZE/2;
	
	free(title);
	free(efg);
	free(dfg);
	free(bg);
	free(bcolor);
	free(func);
	
	return TRUE;
}

boolean write_interface(BigString fname)
{
	FILE *fptr;
	IEMessage *nextmessage;
	IEFunction *nextfunction;
	IEWindow *nextwindow;
	IEPopup *nextpopup;
	
	if ((fptr = GSopen(fname, "wc")) == NULL)	{
		error("Can't open file %s.\n", fname);
		return(FALSE);
		}
		
	fprintf(fptr, "#\tThis interface was generated by the GIL Graphical Builder.\n\n");
	
/*	+++ Use gilheader file instead of this. */
	
	write_palette(fptr);

	write_aliases(fptr);

	write_highlight(fptr);
	
	write_speedup(fptr);
	
	write_buttonstuff(fptr);
	
	write_colorlist(fptr);
	
	write_redrawfunc(fptr);
	fprintf(fptr, "\n");

	nextmessage = iestate.msglisthead;
	while (nextmessage != NULL)	{
		write_message(fptr, nextmessage);
		nextmessage = nextmessage->nextmsg;
		}
		
	fprintf(fptr, "\n");
	nextfunction = iestate.funclisthead;
	while (nextfunction != NULL)	{
		write_function(fptr, nextfunction);
		nextfunction = nextfunction->nextfunc;
		}
	if (iestate.funclisthead != NULL)
		fprintf(fptr, "\n");

	nextpopup = iestate.popuplisthead;
	while (nextpopup != NULL)	{
		write_popup(fptr, nextpopup);
		nextpopup = nextpopup->nextpopup;
		}

	nextwindow = iestate.winlisthead;
	while (nextwindow != NULL)	{
		write_window(fptr, nextwindow);
		nextwindow = nextwindow->nextwin;
		}
	fclose(fptr);
	return(TRUE);
}

void write_palette(FILE *fptr)
{
	int i;

	fprintf(fptr, "PALETTE\n");
	
	for (i=0; (IEColorlist[i] != NULL); i++)	
		if (*IEColorlist[i] != '\0')
			fprintf(fptr, "(%s,\t%d,\t%d,\t%d)\n", IEColorlist[i],
					IEInterface_Palette[3*i], IEInterface_Palette[3*i+1], 
											IEInterface_Palette[3*i+2]);
	fprintf(fptr, "END\n\n");
}

void write_aliases(FILE *fptr)
{
	int i=0;
	
	while(IEAlias[i] != NULL)	{
		fprintf(fptr, "ALIAS (%s, %s)\n", IEAlias[i], IEAcolor[i]);
		if (++i > MAXALIASES)
			return;
		}
	if (IEAlias[0] != NULL)
		fprintf(fptr, "\n");
}

void write_highlight(FILE *fptr)
{
	if (*iestate.highlight != '\0')
		fprintf(fptr, "HIGHLIGHT (%s)\n", iestate.highlight);
}

void write_speedup(FILE *fptr)
{
	if (iestate.speedup >= 0)
		fprintf(fptr, "SPEEDUP (%d)\n", iestate.speedup);
}

void write_buttonstuff(FILE *fptr)
{
	if (*iestate.buttonborder == '\0')
		return;
	
	fprintf(fptr, "BUTTONSTUFF (%s, %d, %s, %s)\n", iestate.buttonborder,
		iestate.diagonals, iestate.buttonTLcolor, iestate.buttonBRcolor);
}

void write_colorlist(FILE *fptr)
{
	int i, j;
	
	for (i=0; i<5; i++)	{
		if (*iestate.colorlist[i][0] == '\0')
			return;
		fprintf(fptr, "COLORLIST (");
		for (j=0; j<5; j++) {
			fprintf(fptr, "%s", iestate.colorlist[i][j]);
			if (j < 4)
				fprintf(fptr, ", ");
			}
		fprintf(fptr, ")\n");
		}
}

void write_redrawfunc(FILE *fptr)
{
	if (*iestate.redrawfunc == '\0')
		return;
		
	fprintf(fptr, "REDRAWFUNC (%s)\n", iestate.redrawfunc);
}

void write_message(FILE *fptr, IEMessage *nextmessage)
{
	fprintf(fptr, "MSG (%s, |%s|)\n", nextmessage->msg.name, nextmessage->msg.text);
}

void write_function(FILE *fptr, IEFunction *nextfunction)
{
/*	fprintf(fptr, "FUNCTION ( %s %d %d %d )\n", nextfunction->name, 
					nextfunction->dtime, nextfunction->active, (int) nextfunction->mouseoff);*/
					
	fprintf(fptr, "FUNCTION (%s, %d, %d", nextfunction->name, 
					nextfunction->dtime, nextfunction->active);
					
	if (nextfunction->mouseoff >= 0)
		fprintf(fptr, ", %d", nextfunction->mouseoff);
		
	fprintf(fptr, ")\n");
}

void write_popup(FILE *fptr, IEPopup *nextpopup)
{
	IEMenuItem *nextmenuitem;
	
/*	fprintf(fptr, "POPUP (%s, %d, %d, %s, %s, %s, %s, %d, %s)\n", nextpopup->title, 
				nextpopup->x, nextpopup->y, nextpopup->efg, nextpopup->dfg, 
				nextpopup->bg, nextpopup->bcolor, (int) nextpopup->checklist,
				nextpopup->function);
*/	
	fprintf(fptr, "POPUP (%s, %d, %d, %s, %s, %s, %s, %d"/*, %s)\n"*/, nextpopup->title, 
				nextpopup->x, nextpopup->y, nextpopup->efg, nextpopup->dfg, 
				nextpopup->bg, nextpopup->bcolor, (int) nextpopup->checklist/*,
				nextpopup->function*/);
	
	if (strlen(nextpopup->function) > 0)
		fprintf(fptr, ", %s", nextpopup->function);
	
	fprintf(fptr, ")\n");
	
	nextmenuitem = nextpopup->itemlisthead;
	while (nextmenuitem != NULL)	{
		write_menu_item(fptr, nextmenuitem);
		nextmenuitem = nextmenuitem->nextmenuitem;
		}
	fprintf(fptr, "\n");
}

void write_menu_item(FILE *fptr, IEMenuItem *nextmenuitem)
{
	fprintf(fptr, "\tPLABEL (%s, %d, %s, |%s|", nextmenuitem->name,
				(int) nextmenuitem->active, nextmenuitem->funcname,	
												nextmenuitem->text);

	/*if (nextmenuitem->checked >= 0)
		fprintf(fptr, ", %d", nextmenuitem->checked);*/
	
	fprintf(fptr, ")\n");
}

void write_window(FILE *fptr, IEWindow *nextwindow)
{
	IEButton *nextbutton;
	IEDragArea *nextdragarea;
	IEGraphic *nextgraphic;
	IEField *nextfield;
	
	fprintf(fptr, "WINDOW (%s, %d, %d, %d, %d, %s, %s, %s, %d, %d, %d)\n", nextwindow->name,
			nextwindow->x, nextwindow->y, nextwindow->wd, nextwindow->ht,
			nextwindow->fgcol, nextwindow->bgcol, nextwindow->bdrcol,
			nextwindow->bordersize, (int) nextwindow->active, nextwindow->bstore_to_use);
	
	nextbutton = nextwindow->firstbutton;
	while (nextbutton != NULL)	{
		write_button(fptr, nextbutton);
		nextbutton = nextbutton->nextbutton;
		}
		
	nextdragarea = nextwindow->firstdragarea;
	while (nextdragarea != NULL)	{
		write_dragarea(fptr, nextdragarea);
		nextdragarea = nextdragarea->nextdragarea;
		}
		
	nextgraphic = nextwindow->firstgraphic;
	while (nextgraphic != NULL)	{
		write_graphic(fptr, nextgraphic);
		nextgraphic = nextgraphic->nextgraphic;
		}
		
	nextfield = nextwindow->firstfield;
	while (nextfield != NULL)	{
		write_field(fptr, nextfield);
		nextfield = nextfield->nextfield;
		}
	fprintf(fptr, "\n");
}

void write_button(FILE *fptr, IEButton *nextbutton)
{
/*	if (nextbutton->tsize == LARGE)	{
		fprintf(fptr, "\tBUTTON ( %s |%s| %d %d %d %d %s %s %s %s %d %d %d %c )\n",
			nextbutton->name, nextbutton->text, nextbutton->x, nextbutton->y,
			nextbutton->wd, nextbutton->ht, nextbutton->efgcolorname, 
			nextbutton->dfgcolorname, nextbutton->bgcolor, nextbutton->funcname,
			(int) nextbutton->border, (int) nextbutton->active,
			(int) nextbutton->repeatable, nextbutton->textjust);
		}
	else	{
		fprintf(fptr, "\tBUTTONS ( %s |%s| %d %d %d %d %s %s %s %s %d %d %d %s )\n",
			nextbutton->name, nextbutton->text, nextbutton->x, nextbutton->y,
			nextbutton->wd, nextbutton->ht, nextbutton->efgcolorname, 
			nextbutton->dfgcolorname, nextbutton->bgcolor, nextbutton->funcname,
			(int) nextbutton->border, (int) nextbutton->active,
			(int) nextbutton->repeatable, nextbutton->textjust);
		}*/
	fprintf(fptr, "\t%s (%s, |%s|, %d, %d, %d, %d, %s, %s, "
					"%s, %s, %d, %d, %d, %c)\n",
		(nextbutton->tsize == LARGE)? "BUTTON" : "BUTTONS",
		nextbutton->name, nextbutton->text, nextbutton->x, nextbutton->y,
		nextbutton->wd, nextbutton->ht, nextbutton->efgcolorname, 
		nextbutton->dfgcolorname, nextbutton->bgcolor, nextbutton->funcname,
		(int) nextbutton->border, (int) nextbutton->active,
		(int) nextbutton->repeatable, nextbutton->textjust);
}

void write_dragarea(FILE *fptr, IEDragArea *nextdragarea)
{
	fprintf(fptr, "\tDRAGAREA (%s, %d, %d, %d, %d, %s, %s, %d)\n", nextdragarea->name,
		nextdragarea->x, nextdragarea->y, nextdragarea->wd, nextdragarea->ht,
		nextdragarea->col, nextdragarea->funcname, (int) nextdragarea->active);
}

void write_graphic(FILE *fptr, IEGraphic *nextgraphic)
{
	boolean shadow;
	
	switch (nextgraphic->gr.type)	{
	
		case GR_LINE:
		case GR_WLINE:
			fprintf(fptr, "\tLINE (%d, %d, %d, %d, %d, %s)\n", nextgraphic->gr.x,
				nextgraphic->gr.y, nextgraphic->gr.wd, nextgraphic->gr.ht,
				(nextgraphic->gr.type == GR_LINE)? 1 : 2, nextgraphic->grcol);
			break;
			
		case GR_RECT:
		case GR_WRECT:
			fprintf(fptr, "\tRECT (%d, %d, %d, %d, %s, %d, %s)\n", nextgraphic->gr.x,
				nextgraphic->gr.y, nextgraphic->gr.wd, nextgraphic->gr.ht,
				nextgraphic->grcol, (nextgraphic->gr.type == GR_RECT)? 1 : 2, 
				(nextgraphic->gr.attrib == FILL)? "FILL" : "HOLLOW");
			break;

		case GR_LABEL:
		case GR_LABEL_SMALL:
			shadow = (strcmp(nextgraphic->grshadowcol, "NONE") != 0);
			fprintf(fptr, "\t%s (%d, %d, %s, |%s|, %s)\n", 
				(nextgraphic->gr.type == GR_LABEL)? "LABEL" : "LABELS", 
				nextgraphic->gr.x, nextgraphic->gr.y+nextgraphic->gr.ht, 
					nextgraphic->grcol, nextgraphic->gr.text, 
					(shadow)? nextgraphic->grshadowcol : "");
			break;

		default:
			break;
		}
}

void write_field(FILE *fptr, IEField *nextfield)
{
	fprintf(fptr, "\tFIELD (%s, %d, %d, %d, %d, %s)\n", nextfield->name, nextfield->x,
					nextfield->y, nextfield->wd, nextfield->ht, nextfield->col);
}

void close_file()
{
	IEMessage *m, *n;
	IEFunction *f, *g;
	IEPopup *p, *q;
	IEWindow *w, *x;
	
	m = iestate.msglisthead;	
	while (m != NULL)	{
		free (m->msg.name);
		free (m->msg.text);
		n = m;
		m = m->nextmsg;
		free (n);
		}
			
	f = iestate.funclisthead;
	while (f != NULL)	{
		g = f;
		f = f->nextfunc;
		free (g);
		}

	p = iestate.popuplisthead;
	while (p != NULL)	{
		free_popupitems(p);
		q = p;
		p = p->nextpopup;
		free (q);
		}

	w = iestate.winlisthead;
	while (w != NULL)	{
		free_buttons(w);
		free_dragareas(w);
		free_graphics(w);
		free_fields(w);
		x = w;
		w = w->nextwin;
		free (x);
		}
}

void free_popupitems(IEPopup *p)
{
	IEMenuItem *i, *j;
	
	i = p->itemlisthead;
	while (i != NULL)	{
		j = i;
		i = i->nextmenuitem;
		free (j);
		}
}

void free_buttons(IEWindow *w)
{
	IEButton *b, *c;
	
	b = w->firstbutton;
	while (b != NULL)	{
		c = b;
		b = b->nextbutton;
		free (c);
		}
}

void free_dragareas(IEWindow *w)
{
	IEDragArea *d, *e;
	
	d = w->firstdragarea;
	while (d != NULL)	{
		e = d;
		d = d->nextdragarea;
		free (e);
		}
}

void free_graphics(IEWindow *w)
{
	IEGraphic *g, *h;
	
	g = w->firstgraphic;
	while (g != NULL)	{
		if ((g->gr.type == GR_LABEL) || (g->gr.type == GR_LABEL_SMALL))	
			free (g->gr.text);
		h = g;
		g = g->nextgraphic;
		free (h);
		}
}

void free_fields(IEWindow *w)
{
	IEField *f, *g;
	
	f = w->firstfield;
	while (f != NULL)	{
		g = f;
		f = f->nextfield;
		free (g);
		}
}

void error(char *message, char *info)
{
	GSbell();
	GSelog(0, "Error in edit file.\n%s\n%s\n", message, info);
	fileOpenSuccess = FALSE;
}

void ieerror(char *msg)
{
	error(msg, ieLineBuf);
}
