#include <stdlib.h>
#include <string.h>
#include "geosim.h"
#include "gspriv.h"
#include "ietypes.h"
#include "messagew.h"
#include "windfunc.h"
#include "editrec.h"
/*#include "rootw.h"*/
#include "iemisc.h"
#include "helpmsg.h"

extern boolean messagewind;
extern IEIconfig iestate;

static GSlistHandle messageList;
static GSsclistHandle messageScroll;
static int numMsg, curMsg;
static HugeString tempMessageText, tempMessageName;
static EditRec *messageTextEdit=NULL, *messageNameEdit=NULL;

/* internal protos */
void SetupMessageList();
char *MessageLabel(int);
void ClearMessageField();
IEMessage *GetMessageNum(int);
char *mLabelFunc(int slot);
void mTextFunc(GSlistHandle, int slot);
void mInFunc() ;
void mOutFunc() ;
void messageChanged(); 
void mClickFunc(int slot);
/**/


/* pick list functions */

char *mLabelFunc(int slot)
{
	if (slot <= numMsg)
		return MessageLabel(slot);
	else
		return "";
}

void mTextFunc(GSlistHandle lh, int slot)
{
	if (slot == curMsg)
		GSsetcolor(GSconvertcolor("red"));
	else
		GSsetcolor(GSconvertcolor("black"));
	GSsetfont(SMALL);
}

void mInFunc() 
{
}

void mOutFunc() 
{
}

void messageChanged() /* notification func */
{
}

void mClickFunc(int slot)
{
	IEMessage *theMsg;
	
	int old = curMsg;
	
	if ((curMsg >= 0)&&(tempMessageText[0] != '\0')
					&&(tempMessageName[0] != '\0')) {
		/* copy contents of tempMessageText[] to current msg text */
		theMsg = GetMessageNum(old);
		if (theMsg) {
			free(theMsg->msg.text);
			theMsg->msg.text = str_dup(tempMessageText);
			free(theMsg->msg.name);
			theMsg->msg.name = str_dup(tempMessageName);
			}
		}
	
	ClearMessageField();
	
	curMsg = slot;
	
	theMsg = GetMessageNum(slot);
	GSdrawlistitem(messageList, New, curMsg);
	
	/* enable del btn */
	strcpy(tempMessageText, theMsg->msg.text);
	strcpy(tempMessageName, theMsg->msg.name);
	messageTextEdit->inspt = messageNameEdit->inspt = 0;
	UpdateEditRec(messageTextEdit);
	UpdateEditRec(messageNameEdit);
	
	GSenablebutton("messagew", "DelMsg", TRUE);
}

void DrawMessageWind()
{
	messagewind = TRUE;
	GSdrawnamedwindow("messagew", NO_BSTORE, NO_POPUP);
	GSsetcurrentnamedwindow("messagew");
	GSdisablebutton("messagew", "DelMsg", TRUE);
	
	tempMessageName[0] = tempMessageText[0] = '\0';
	
	messageNameEdit = NewEditRec(tempMessageName, BIGSTRINGLEN, 1, SMALL,
					"messagew", "msg_edit_name", c_identifier_filter, messageChanged);
	messageTextEdit = NewEditRec(tempMessageText, BIGSTRINGLEN, 1, SMALL,
					"messagew", "msg_edit_text", NULL, messageChanged);
	SetCurrentEditRec(messageTextEdit);
	UpdateEditRec(messageNameEdit);
	UpdateEditRec(messageTextEdit);
	
	GSdisablemenuitem("Edit", "edit_messages");
	SetupMessageList();
	GSdisplayhelpmessage(MSG_H);
	ShowVersion();
}

void RedrawMessageWind()
{
	int x, y, wd, ht;

	GSredrawnamedwindow("messagew");
	UpdateEditRec(messageNameEdit);
	UpdateEditRec(messageTextEdit);
	GSdrawsclist(messageScroll);
	GSgetdragareaspecs("messagew", "msgpick", &x, 
								&y, &wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);
}

void RemoveMessageWind()
{
	IEMessage *theMsg;
	
	if ((curMsg >= 0)/*&&(tempMessageText[0] != '\0')*/
					&&(tempMessageName[0] != '\0')) {
		/* copy contents of tempMessageText[] to current msg text */
		theMsg = GetMessageNum(curMsg);
		if (theMsg) {
			free(theMsg->msg.text);
			theMsg->msg.text = str_dup(tempMessageText);
			free(theMsg->msg.name);
			theMsg->msg.name = str_dup(tempMessageName);
			}
		}

	messagewind = FALSE;
	GSdismantlesclist(messageScroll);
	GSremovenamedwindow("messagew");
	DisposEditRec(messageNameEdit);
	DisposEditRec(messageTextEdit);
	messageTextEdit = messageNameEdit = NULL;
	GSenablemenuitem("Edit", "edit_messages");
}

void SetupMessageList()
{
	int x, y, wd, ht;
	
	IEMessage *cur = iestate.msglisthead;
	
	numMsg = 0;
	while (cur != NULL) {
		numMsg++;
		cur = cur->nextmsg;
		}
	GSgetdragareaspecs("messagew", "msgpick", &x, 
								&y, &wd, &ht, NULL, NULL);
	
	curMsg = NO_ITEM;
	
	GSsclistsetup("messagew", "msgpick",
					1,
					"picklabel",
					0, 0,
					JUST_LEFT,
					mLabelFunc,
					mTextFunc,
					mInFunc,
					mOutFunc,
					mClickFunc,
					"msgscroll",
					GSconvertcolor("sclistcol"),
					GSconvertcolor("sclisttl"),
					GSconvertcolor("sclistbr"),
					numMsg, 0,
					NULL,
					"MUpArrow", "MDownArrow",
					&messageList, &messageScroll);
					
	ShadowRect(x, y, wd, ht, THIN, Concave);
	GSdrawsclist(messageScroll);
}

char *MessageLabel(int which)
{
	IEMessage *m = GetMessageNum(which);
	
	return (m==NULL)? "" : m->msg.name;
}

IEMessage *GetMessageNum(int num)
{
	IEMessage *cur = iestate.msglisthead;

	if (num < 0)
		return NULL;

	while (cur != NULL) {
		if (num == 0)
			return cur;
		num--;
		cur = cur->nextmsg;
		}
	
	return NULL;
}

void ClearMessageField()
{
	int old;
	old = curMsg;
	curMsg = -1;
	
	if (old >= 0)
		GSdrawlistitem(messageList, New, old);
		
	tempMessageText[0] = '\0';
	messageTextEdit->inspt = 0;
	UpdateEditRec(messageTextEdit);
	tempMessageName[0] = '\0';
	messageNameEdit->inspt = 0;
	UpdateEditRec(messageNameEdit);

	GSenablebutton("messagew", "DelMsg", TRUE);
}

boolean message_up()
{
	GShandleupbutton(messageScroll);
	return TRUE;
}

boolean message_down()
{
	GShandledownbutton(messageScroll);
	return TRUE;
}

boolean message_scroll(int relx, int rely, DragStatusType status)
{
	GShandlesclist(messageScroll, relx, rely, status);
}

boolean message_pick(int relx, int rely, DragStatusType status)
{
	GShandlelist(messageList, relx, rely, status);
}

boolean edit_message_name(int x, int y, DragStatusType status)
{
	if (messageNameEdit == NULL)
		return FALSE;
	if ((status == MOUSE_CLICK) || (status == DRAG_FINAL))
		SetCurrentEditRec(messageNameEdit);
}

boolean edit_message_text(int x, int y, DragStatusType status)
{
	if (messageTextEdit == NULL)
		return FALSE;
	if ((status == MOUSE_CLICK) || (status == DRAG_FINAL))
		SetCurrentEditRec(messageTextEdit);
}

boolean new_message(void)
{
	/* If there's a current message with content,
	copy into the message.  Else, if content, make
	new message.  Finally, clear fields.
	*/
	IEMessage *theMsg;
	int old = curMsg;
	
	if ((curMsg >= 0)&&(tempMessageText[0] != '\0')
					&&(tempMessageName[0] != '\0')) {
		/* copy contents of tempMessageText[] to current msg text */
		theMsg = GetMessageNum(old);
		if (theMsg) {
			free(theMsg->msg.text);
			theMsg->msg.text = str_dup(tempMessageText);
			free(theMsg->msg.name);
			theMsg->msg.name = str_dup(tempMessageName);
			}
		/* GSsetsclistnumitems(messageScroll, ++numMsg, TRUE);  
								whoa - what's this doing here???*/
		}
	else if ((tempMessageText[0] != '\0') &&
					(tempMessageName[0] != '\0'))
		{
		theMsg = (IEMessage *) malloc (sizeof(IEMessage));
		theMsg->msg.text = str_dup(tempMessageText);
		theMsg->msg.name = str_dup(tempMessageName);
		theMsg->nextmsg = NULL;
		/* add to end */
		
		if (iestate.msglisttail == NULL) 
			iestate.msglisttail = iestate.msglisthead = theMsg;	
		else {
			iestate.msglisttail->nextmsg = theMsg;
			iestate.msglisttail = theMsg;
			}
		GSsetsclistnumitems(messageScroll, ++numMsg, TRUE);
		}
		
		
	ClearMessageField();
}

boolean delete_message(void)
{
	IEMessage *prev = NULL, *cur;
	
	if (curMsg < 0) {
		GSdisablebutton("messagew", "DelMsg", TRUE);
		return;
		/* shouldn't be here in the first place */
		}
	
	cur = GetMessageNum(curMsg);
	
	if (curMsg == 0)
		iestate.msglisthead = cur->nextmsg;
	else {
		prev = GetMessageNum(curMsg-1);
		prev->nextmsg = cur->nextmsg;
		}
	
	if (iestate.msglisttail == cur)
		iestate.msglisttail = prev;
		
	free(cur->msg.text);
	free(cur->msg.name);
	free(cur);
	GSsetsclistnumitems(messageScroll, --numMsg, TRUE);
	ClearMessageField();
}